@@ -15,12 +15,11 @@
 #include "GISel/M68kCallLowering.h"
 #include "GISel/M68kLegalizerInfo.h"
 #include "GISel/M68kRegisterBankInfo.h"
-
 #include "M68k.h"
 #include "M68kMachineFunction.h"
 #include "M68kRegisterInfo.h"
+#include "M68kSelectionDAGInfo.h"
 #include "M68kTargetMachine.h"
-
 #include "llvm/CodeGen/MachineJumpTableInfo.h"
 #include "llvm/IR/Attributes.h"
 #include "llvm/IR/Function.h"
@@ -50,10 +49,12 @@ void M68kSubtarget::anchor() {}
 
 M68kSubtarget::M68kSubtarget(const Triple &TT, StringRef CPU, StringRef FS,
                              const M68kTargetMachine &TM)
-    : M68kGenSubtargetInfo(TT, CPU, /*TuneCPU*/ CPU, FS), TM(TM), TSInfo(),
+    : M68kGenSubtargetInfo(TT, CPU, /*TuneCPU*/ CPU, FS), TM(TM),
       InstrInfo(initializeSubtargetDependencies(CPU, TT, FS, TM)),
       FrameLowering(*this, this->getStackAlignment()), TLInfo(TM, *this),
       TargetTriple(TT) {
+  TSInfo = std::make_unique<M68kSelectionDAGInfo>();
+
   CallLoweringInfo.reset(new M68kCallLowering(*getTargetLowering()));
   Legalizer.reset(new M68kLegalizerInfo(*this));
 
@@ -62,6 +63,12 @@ M68kSubtarget::M68kSubtarget(const Triple &TT, StringRef CPU, StringRef FS,
   InstSelector.reset(createM68kInstructionSelector(TM, *this, *RBI));
 }
 
+M68kSubtarget::~M68kSubtarget() = default;
+
+const SelectionDAGTargetInfo *M68kSubtarget::getSelectionDAGInfo() const {
+  return TSInfo.get();
+}
+
 const CallLowering *M68kSubtarget::getCallLowering() const {
   return CallLoweringInfo.get();
 }
@@ -115,7 +122,7 @@ M68kSubtarget &M68kSubtarget::initializeSubtargetDependencies(
 //  ---------------------+------------+------------+------------+-------------
 //                branch |   pc-rel   |   pc-rel   |   pc-rel   |   pc-rel
 //  ---------------------+------------+------------+------------+-------------
-//           call global |    @PLT    |    @PLT    |    @PLT    |    @PLT
+//           call global |  absolute  |    @PLT    |  absolute  |    @PLT
 //  ---------------------+------------+------------+------------+-------------
 //         call internal |   pc-rel   |   pc-rel   |   pc-rel   |   pc-rel
 //  ---------------------+------------+------------+------------+-------------
@@ -127,6 +134,24 @@ M68kSubtarget &M68kSubtarget::initializeSubtargetDependencies(
 //  ---------------------+------------+------------+------------+-------------
 //      data global big* |   pc-rel   |  @GOTPCREL |  absolute  |  @GOTPCREL
 //  ---------------------+------------+------------+------------+-------------
+//                       |          Large          |
+//                       +-------------------------+
+//                       |   Static   |    PIC     |
+//  ---------------------+------------+------------+
+//                branch |  absolute  |   pc-rel   |
+//  ---------------------+------------+------------+
+//           call global |  absolute  |    @PLT    |
+//  ---------------------+------------+------------+
+//         call internal |  absolute  |   pc-rel   |
+//  ---------------------+------------+------------+
+//            data local |  absolute  |  @GOTOFF   |
+//  ---------------------+------------+------------+
+//       data local big* |  absolute  |  @GOTOFF   |
+//  ---------------------+------------+------------+
+//           data global |  absolute  |  @GOTOFF   |
+//  ---------------------+------------+------------+
+//      data global big* |  absolute  |  @GOTOFF   |
+//  ---------------------+------------+------------+
 //
 // * Big data potentially cannot be reached within 16 bit offset and requires
 //   special handling for old(x00 and x10) CPUs. Normally these symbols go into
@@ -142,8 +167,22 @@ M68kSubtarget &M68kSubtarget::initializeSubtargetDependencies(
 /// Classify a blockaddress reference for the current subtarget according to how
 /// we should reference it in a non-pcrel context.
 unsigned char M68kSubtarget::classifyBlockAddressReference() const {
-  // Unless we start to support Large Code Model branching is always pc-rel
-  return M68kII::MO_PC_RELATIVE_ADDRESS;
+  switch (TM.getCodeModel()) {
+  default:
+    llvm_unreachable("Unsupported code model");
+  case CodeModel::Small:
+  case CodeModel::Kernel:
+  case CodeModel::Medium: {
+    return M68kII::MO_PC_RELATIVE_ADDRESS;
+  }
+  case CodeModel::Large: {
+    if (isPositionIndependent()) {
+      return M68kII::MO_PC_RELATIVE_ADDRESS;
+    } else {
+      return M68kII::MO_ABSOLUTE_ADDRESS;
+    }
+  }
+  }
 }
 
 unsigned char
@@ -171,11 +210,18 @@ M68kSubtarget::classifyLocalReference(const GlobalValue *GV) const {
       return M68kII::MO_ABSOLUTE_ADDRESS;
     }
   }
+  case CodeModel::Large: {
+    if (isPositionIndependent()) {
+      return M68kII::MO_GOTOFF;
+    } else {
+      return M68kII::MO_ABSOLUTE_ADDRESS;
+    }
+  }
   }
 }
 
 unsigned char M68kSubtarget::classifyExternalReference(const Module &M) const {
-  if (TM.shouldAssumeDSOLocal(M, nullptr))
+  if (TM.shouldAssumeDSOLocal(nullptr))
     return classifyLocalReference(nullptr);
 
   if (isPositionIndependent())
@@ -191,7 +237,7 @@ M68kSubtarget::classifyGlobalReference(const GlobalValue *GV) const {
 
 unsigned char M68kSubtarget::classifyGlobalReference(const GlobalValue *GV,
                                                      const Module &M) const {
-  if (TM.shouldAssumeDSOLocal(M, GV))
+  if (TM.shouldAssumeDSOLocal(GV))
     return classifyLocalReference(GV);
 
   switch (TM.getCodeModel()) {
@@ -212,6 +258,12 @@ unsigned char M68kSubtarget::classifyGlobalReference(const GlobalValue *GV,
 
     return M68kII::MO_ABSOLUTE_ADDRESS;
   }
+  case CodeModel::Large: {
+    if (isPositionIndependent())
+      return M68kII::MO_GOTOFF;
+
+    return M68kII::MO_ABSOLUTE_ADDRESS;
+  }
   }
 }
 
@@ -221,7 +273,8 @@ unsigned M68kSubtarget::getJumpTableEncoding() const {
     // the potential delta between the jump target and table base can be larger
     // than displacement field, which is True for older CPUs(16 bit disp)
     // in Medium model(can have large data way beyond 16 bit).
-    if (TM.getCodeModel() == CodeModel::Medium && !atLeastM68020())
+    if ((TM.getCodeModel() == CodeModel::Medium && !atLeastM68020()) ||
+        TM.getCodeModel() == CodeModel::Large)
       return MachineJumpTableInfo::EK_Custom32;
 
     return MachineJumpTableInfo::EK_LabelDifference32;
@@ -240,7 +293,7 @@ unsigned char
 M68kSubtarget::classifyGlobalFunctionReference(const GlobalValue *GV,
                                                const Module &M) const {
   // local always use pc-rel referencing
-  if (TM.shouldAssumeDSOLocal(M, GV))
+  if (TM.shouldAssumeDSOLocal(GV))
     return M68kII::MO_NO_FLAG;
 
   // If the function is marked as non-lazy, generate an indirect call
@@ -251,6 +304,6 @@ M68kSubtarget::classifyGlobalFunctionReference(const GlobalValue *GV,
     return M68kII::MO_GOTPCREL;
   }
 
-  // otherwise linker will figure this out
-  return M68kII::MO_PLT;
+  // Ensure that we don't emit PLT relocations when in non-pic modes.
+  return isPositionIndependent() ? M68kII::MO_PLT : M68kII::MO_ABSOLUTE_ADDRESS;
 }
